@@ -73,6 +73,9 @@ def test_add_to_perioddata(shellmound_sfrdata):
                         for k, v in sfr_routing.items()}
     nlines = 4
     seq = add_line_sequence(flowline_routing, nlines=nlines)
+    
+    # two inflows applied upstream of model on same path;
+    # results should be summed an applied to first line_id in model
     flows = pd.DataFrame({'Q_avg': [100., 10., 200., 20.],
                           'per': [0, 1, 0, 1],
                           'line_id': [2, 2, 4, 4]})
@@ -81,13 +84,78 @@ def test_add_to_perioddata(shellmound_sfrdata):
                       variable='inflow',
                       line_id_column='line_id',
                       period_column='per',
-                      data_column='Q_avg')
-    flows = flows.loc[flows.line_id != 2]
+                      data_column='Q_avg',
+                      one_inflow_per_path=False)
+    flows = flows.groupby('per').sum()
     assert np.allclose(sfrd.period_data['inflow'].values, flows['Q_avg'].values)
-    assert np.allclose(sfrd.period_data['per'].values, flows['per'].values)
-    assert rd.loc[rd.line_id == seq[-1], 'rno'].values[0] == sfrd.period_data['rno'].values[0]
+    assert np.allclose(sfrd.period_data.index.get_level_values(0), 
+                       flows.index.values)
+    assert rd.loc[rd.line_id == seq[-1], 'rno'].values[0] == sfrd.period_data.index.levels[1].values[0]
+    
+    # two inflows applied upstream of model on different paths;
+    # inflows should be applied to the first lines in the model 
+    # along their respective paths
+    flowline_routing[6] = 1000005
+    flows = pd.DataFrame({'Q_avg': [100., 10., 200., 20.],
+                          'per': [0, 1, 0, 1],
+                          'line_id': [6, 6, 4, 4]})
+    add_to_perioddata(sfrd, flows,
+                      flowline_routing=flowline_routing,
+                      variable='inflow',
+                      line_id_column='line_id',
+                      period_column='per',
+                      data_column='Q_avg',
+                      one_inflow_per_path=False)
+    assert np.allclose(sfrd.period_data['inflow'].sort_values(), flows['Q_avg'].sort_values())
+    assert np.allclose(sfrd.period_data.index.get_level_values(0), 
+                       sorted(flows['per']))
+    lines = {flowline_routing[6], seq[-1]}
+    rnos = rd.loc[rd.line_id.isin(lines), 'rno']
+    assert not set(sfrd.period_data.index.levels[1]).difference(rnos)
+    assert len(set(sfrd.period_data.index.levels[1])) == 2
 
+    # two inflows applied along same path in model;
+    # one_inflow_per_path=True
+    # inflows should be summed and applied to most downstream reach
+    # test updating of period_data
+    # sfrd.period_data wasn't reset, so should contain
+    # updated values of 300 and 30 at reach 354
+    # and existing values of 100 and 10 at 395
+    flows = pd.DataFrame({'Q_avg': [100., 10., 200., 20.],
+                          'per': [0, 1, 0, 1],
+                          'line_id': [2, 2, 4, 4]})
+    add_to_perioddata(sfrd, flows,
+                      flowline_routing=flowline_routing,
+                      variable='inflow',
+                      line_id_column='line_id',
+                      period_column='per',
+                      data_column='Q_avg',
+                      one_inflow_per_path=True)
+    assert np.allclose(sfrd.period_data['inflow'].values, [300., 100., 30., 10.])
+    assert np.allclose(sfrd.period_data.index.levels[1], [354, 394])
 
+    # two inflows applied along same path in model;
+    # one_inflow_per_path=False
+    # inflows should be applied to their respective reaches
+
+    flows = pd.DataFrame({'Q_avg': [100., 10., 200., 20.],
+                          'per': [0, 1, 0, 1],
+                          'line_id': [17955337, 17955337, 
+                                      flowline_routing[17955337], 
+                                      flowline_routing[17955337]
+                                      ]})
+    sfrd._period_data = None
+    add_to_perioddata(sfrd, flows,
+                      flowline_routing=flowline_routing,
+                      variable='inflow',
+                      line_id_column='line_id',
+                      period_column='per',
+                      data_column='Q_avg',
+                      one_inflow_per_path=False)
+    assert np.allclose(sfrd.period_data['inflow'].values, [100., 10., 200., 20.])
+    assert np.allclose(sfrd.period_data.index.levels[1], [354, 401])
+    
+    
 def test_add_to_segment_data(shellmound_sfrdata):
     sfrd = shellmound_sfrdata  # copy.deepcopy(shellmound_sfrdata)
     rd = shellmound_sfrdata.reach_data.copy()
