@@ -1,4 +1,4 @@
-import collections
+from collections.abc import Mapping
 import os
 import inspect
 from pathlib import Path
@@ -282,8 +282,8 @@ def update(d, u):
     from: https://stackoverflow.com/questions/3232943/update-value-of-a-nested-dictionary-of-varying-depth
     """
     for k, v in u.items():
-        if isinstance(d, collections.Mapping):
-            if isinstance(v, collections.Mapping):
+        if isinstance(d, Mapping):
+            if isinstance(v, Mapping):
                 d[k] = update(d.get(k, {}), v)
             else:
                 d[k] = v
